// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  struct Ray3D {
    glm::vec3 origin;
    glm::vec3 direction;
  };

  /**
   * Tests whether a ray intersects a quad within 3D space.
   * 
   * @param ray Ray to test.
   * @param min Minimum point of the quad.
   * @param max Maximum point of the quad.
   * @param transform Transform of the quad.
   * @param point Out point of intersection.
   * @param normal Out normal of the quad at the point of intersection.
   * @param distance Out distance from the ray origin to the intersection.
   * @return True if the ray intersects the quad, false otherwise.
   */
  bool_t raytestQuad(
    const struct Ray3D ray,
    const glm::vec2 min,
    const glm::vec2 max,
    const glm::mat4 transform,
    glm::vec3 &point,
    glm::vec3 &normal,
    float_t &distance
  );
}
