// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Scene.hpp"
#include "SceneItem.hpp"
#include "SceneItemComponent.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

Scene::Scene(const std::weak_ptr<DawnGame> game) {
  this->game = game;
  this->nextId = 0;
}

void Scene::update() {
  // Finsh adding scene items that were trying to add from the last frame.
  auto it = this->itemsNotInitialized.begin();
  while(it != this->itemsNotInitialized.end()) {
    this->items[it->first] = it->second;
    it->second->init();
    ++it;
  }
  this->itemsNotInitialized.clear();
  
  // TODO: Cleanup old scene items

  // TODO: Tick scene items(?)
  auto game = this->game.lock();
  this->eventSceneUpdate.invoke(game->timeManager.delta);
  if(!game->timeManager.isPaused) this->eventSceneUnpausedUpdate.invoke(game->timeManager.delta);
}

std::shared_ptr<SceneItem> Scene::createSceneItem() {
  return this->createSceneItemOfType<SceneItem>();
}

Scene::~Scene() {
  // Invoke cleanup.
  auto it = this->items.begin();
  while(it != this->items.end()) {
    it->second->destroy();
    ++it;
  }
}