// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SceneItemComponent.hpp"
#include "SceneItem.hpp"
#include "Scene.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

SceneItemComponent::SceneItemComponent(std::weak_ptr<SceneItem> item) {
  this->item = item;
}

void SceneItemComponent::init() {
  this->onStart();
  this->hasInitialized = true;
}

std::vector<std::shared_ptr<SceneItemComponent>> SceneItemComponent::getDependencies() {
  return {};
}

std::shared_ptr<Scene> SceneItemComponent::getScene() {
  auto scene = this->item.lock()->scene.lock();
  assertNotNull(scene, "Scene cannot be null!");
  return scene;
}

std::shared_ptr<DawnGame> SceneItemComponent::getGame() {
  auto game = this->getScene()->game.lock();
  assertNotNull(game, "Game cannot be null!");
  return game;
}

void SceneItemComponent::onStart() {
  
}

void SceneItemComponent::onDispose() {

}

void SceneItemComponent::onStateUpdate() {
  
}

SceneItemComponent::~SceneItemComponent() {

}