// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "prefabs/SimpleSpinningCubePrefab.hpp"

namespace Dawn {
  class TestScene : public Scene {
    protected:
      Camera *camera;
      SimpleSpinningCubePrefab *cube;

      void stage() override {
        camera = Camera::create(shared_from_this());
        camera->lookAt(glm::vec3(5, 5, 5), glm::vec3(0, 0, 0));

        cube = SimpleSpinningCubePrefab::prefabCreate(this);
      }
      
      std::vector<std::shared_ptr<Asset>> getRequiredAssets() override {
        auto assMan = &this->game->assetManager;
        std::vector<std::shared_ptr<Asset>> assets;
        vectorAppend(assets, SimpleSpinningCubePrefab::getRequiredAssets(assMan));
        return assets;
      }

    public:
      TestScene(DawnGame *game) : Scene(game) {}
  };
}