// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/BackBufferRenderTarget.hpp"
#include "display/shader/ShaderManager.hpp"
#include "display/shader/shaders/SimpleTexturedShader.hpp"
#include "display/shader/shaders/FontShader.hpp"
#include "display/shader/shaders/UIShader.hpp"
#include "display/RenderPipeline.hpp"

namespace Dawn {
  class RenderManager : public IRenderManager {
    private:
      std::shared_ptr<RenderPipeline> renderPipeline;
      std::shared_ptr<ShaderManager> shaderManager;
      shaderlock_t lockSimpleTextured = -1;
      shaderlock_t lockUIShaderProgram = -1;
      shaderlock_t lockFontShader = -1;

    public:
      std::shared_ptr<BackBufferRenderTarget> backBuffer;
      std::shared_ptr<SimpleTexturedShader> simpleTexturedShader;
      std::shared_ptr<UIShader> uiShader;
      std::shared_ptr<FontShader> fontShader;

      /**
       * Construct a new RenderManager for a game instance.
       */
      RenderManager();
      
      std::shared_ptr<RenderTarget> getBackBuffer() override;
      std::shared_ptr<RenderPipeline> getRenderPipeline() override;
      std::shared_ptr<ShaderManager> getShaderManager() override;
      void setRenderFlags(const flag_t renderFlags) override;
      void init(const std::weak_ptr<DawnGame> game) override;
      void update() override;
      
      /**
       * Destroy a previously initialized RenderManager.
       */
      ~RenderManager();
  };
}